package cn.com.infosec.jce.provider;

import cn.com.infosec.asn1.ASN1Encodable;
import cn.com.infosec.asn1.ASN1InputStream;
import cn.com.infosec.asn1.ASN1OutputStream;
import cn.com.infosec.asn1.ASN1Sequence;
import cn.com.infosec.asn1.DEREncodable;
import cn.com.infosec.asn1.DERInteger;
import cn.com.infosec.asn1.DERObjectIdentifier;
import cn.com.infosec.asn1.DEROutputStream;
import cn.com.infosec.asn1.util.ASN1Dump;
import cn.com.infosec.asn1.x509.CRLDistPoint;
import cn.com.infosec.asn1.x509.CRLNumber;
import cn.com.infosec.asn1.x509.CertificateList;
import cn.com.infosec.asn1.x509.IssuingDistributionPoint;
import cn.com.infosec.asn1.x509.TBSCertList;
import cn.com.infosec.asn1.x509.X509Extension;
import cn.com.infosec.asn1.x509.X509Extensions;
import cn.com.infosec.jce.X509Principal;
import cn.com.infosec.util.encoders.Hex;
import cn.com.infosec.x509.extension.X509ExtensionUtil;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.security.Principal;
import java.security.PublicKey;
import java.security.Signature;
import java.security.SignatureException;
import java.security.cert.CRLException;
import java.security.cert.Certificate;
import java.security.cert.X509CRL;
import java.security.cert.X509CRLEntry;
import java.security.cert.X509Certificate;
import java.util.Collections;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import javax.security.auth.x500.X500Principal;

public class X509CRLObject extends X509CRL {
   private CertificateList c;
   private String sigAlgName;
   private byte[] sigAlgParams;
   private boolean isIndirect;

   public X509CRLObject(CertificateList c) {
      CertificateList var10000 = c;
      X509CRLObject var10001 = this;
      CertificateList var10002 = c;
      super();
      this.c = c;

      Exception var7;
      label33: {
         DEREncodable var8;
         boolean var9;
         try {
            var10001.sigAlgName = X509SignatureUtil.getSignatureName(var10002.getSignatureAlgorithm());
            var8 = var10000.getSignatureAlgorithm().getParameters();
         } catch (Exception var5) {
            var7 = var5;
            var9 = false;
            break label33;
         }

         if (var8 != null) {
            try {
               this.sigAlgParams = ((ASN1Encodable)c.getSignatureAlgorithm().getParameters()).getDEREncoded();
            } catch (Exception var4) {
               var7 = var4;
               var9 = false;
               break label33;
            }
         } else {
            try {
               this.sigAlgParams = null;
            } catch (Exception var3) {
               var7 = var3;
               var9 = false;
               break label33;
            }
         }

         try {
            this.isIndirect = this.isIndirectCRL();
            return;
         } catch (Exception var2) {
            var7 = var2;
            var9 = false;
         }
      }

      Exception var6 = var7;
      throw new CRLException("CRL contents invalid: " + var6);
   }

   private Set getExtensionOIDs(boolean critical) {
      X509Extensions var5;
      if (this.getVersion() == 2 && (var5 = this.c.getTBSCertList().getExtensions()) != null) {
         HashSet var2;
         var2 = new HashSet.<init>();
         Enumeration var3 = var5.oids();

         while(var3.hasMoreElements()) {
            DERObjectIdentifier var4;
            if (critical == var5.getExtension(var4 = (DERObjectIdentifier)var3.nextElement()).isCritical()) {
               var2.add(var4.getId());
            }
         }

         return var2;
      } else {
         return null;
      }
   }

   private Set loadCRLEntries() {
      HashSet var1;
      var1 = new HashSet.<init>();
      Enumeration var2 = this.c.getRevokedCertificateEnumeration();

      X509CRLEntryObject var10000;
      for(X500Principal var3 = this.getIssuerX500Principal(); var2.hasMoreElements(); var3 = var10000.getCertificateIssuer()) {
         TBSCertList.CRLEntry var4 = (TBSCertList.CRLEntry)var2.nextElement();
         var10000 = new X509CRLEntryObject(var4, this.isIndirect, var3);
         var1.add(var10000);
      }

      return var1;
   }

   private boolean isIndirectCRL() {
      byte[] var3;
      byte[] var10000 = var3 = this.getExtensionValue(X509Extensions.IssuingDistributionPoint.getId());
      boolean var1 = false;
      if (var10000 != null) {
         boolean var4;
         try {
            var4 = IssuingDistributionPoint.getInstance(X509ExtensionUtil.fromExtensionValue(var3)).isIndirectCRL();
         } catch (Exception var2) {
            throw new ExtCRLException("Exception reading IssuingDistributionPoint", var2);
         }

         var1 = var4;
      }

      return var1;
   }

   public boolean hasUnsupportedCriticalExtension() {
      Set var1;
      if ((var1 = this.getCriticalExtensionOIDs()) == null) {
         return false;
      } else {
         var1.remove(RFC3280CertPathUtilities.ISSUING_DISTRIBUTION_POINT);
         var1.remove(RFC3280CertPathUtilities.DELTA_CRL_INDICATOR);
         return var1.isEmpty() ^ true;
      }
   }

   public Set getCriticalExtensionOIDs() {
      return this.getExtensionOIDs(true);
   }

   public Set getNonCriticalExtensionOIDs() {
      return this.getExtensionOIDs(false);
   }

   public byte[] getExtensionValue(String oid) {
      X509Extensions var3;
      if ((var3 = this.c.getTBSCertList().getExtensions()) != null) {
         X509Extensions var10000 = var3;
         DERObjectIdentifier var4;
         var4 = new DERObjectIdentifier.<init>(oid);
         X509Extension var5;
         if ((var5 = var10000.getExtension(var4)) != null) {
            try {
               return var5.getValue().getEncoded();
            } catch (Exception var2) {
               throw new IllegalStateException("error parsing " + var2.toString());
            }
         }
      }

      return null;
   }

   public byte[] getEncoded() {
      ByteArrayOutputStream var1;
      ByteArrayOutputStream var10000 = var1 = new ByteArrayOutputStream;
      var10000.<init>();
      X509CRLObject var10002 = this;
      DEROutputStream var10001 = new DEROutputStream(var1);

      try {
         var10001.writeObject(var10002.c);
         return var10000.toByteArray();
      } catch (IOException var2) {
         throw new CRLException(var2.toString());
      }
   }

   public void verify(PublicKey key) {
      this.verify(key, "INFOSEC");
   }

   public void verify(PublicKey key, String sigProvider) {
      if (this.c.getSignatureAlgorithm().equals(this.c.getTBSCertList().getSignature())) {
         Signature sigProvider1;
         Signature var10000 = sigProvider1 = Signature.getInstance(this.getSigAlgName(), sigProvider);
         sigProvider1.initVerify(key);
         sigProvider1.update(this.getTBSCertList());
         if (!var10000.verify(this.getSignature())) {
            throw new SignatureException("CRL does not verify with supplied public key.");
         }
      } else {
         throw new CRLException("Signature algorithm on CertificateList does not match TBSCertList.");
      }
   }

   public int getVersion() {
      return this.c.getVersion();
   }

   public Principal getIssuerDN() {
      return new X509Principal(this.c.getIssuer());
   }

   public X500Principal getIssuerX500Principal() {
      ByteArrayOutputStream var10000;
      boolean var10001;
      try {
         var10000 = new ByteArrayOutputStream;
      } catch (IOException var4) {
         var10001 = false;
         throw new IllegalStateException("can't encode issuer DN");
      }

      ByteArrayOutputStream var1 = var10000;

      ASN1OutputStream var5;
      try {
         var10000.<init>();
         var5 = new ASN1OutputStream;
      } catch (IOException var3) {
         var10001 = false;
         throw new IllegalStateException("can't encode issuer DN");
      }

      ASN1OutputStream var6 = var5;

      try {
         var6.<init>(var1);
         var5.writeObject(this.c.getIssuer());
         return new X500Principal(var1.toByteArray());
      } catch (IOException var2) {
         var10001 = false;
         throw new IllegalStateException("can't encode issuer DN");
      }
   }

   public Date getThisUpdate() {
      return this.c.getThisUpdate().getDate();
   }

   public Date getNextUpdate() {
      return this.c.getNextUpdate() != null ? this.c.getNextUpdate().getDate() : null;
   }

   public X509CRLEntry getRevokedCertificate(BigInteger serialNumber) {
      Enumeration var2 = this.c.getRevokedCertificateEnumeration();

      X509CRLEntryObject var5;
      for(X500Principal var3 = this.getIssuerX500Principal(); var2.hasMoreElements(); var3 = var5.getCertificateIssuer()) {
         TBSCertList.CRLEntry var4;
         TBSCertList.CRLEntry var10001 = var4 = (TBSCertList.CRLEntry)var2.nextElement();
         var5 = new X509CRLEntryObject.<init>(var4, this.isIndirect, var3);
         if (serialNumber.equals(var10001.getUserCertificate().getValue())) {
            return var5;
         }
      }

      return null;
   }

   public Set getRevokedCertificates() {
      Set var1;
      return !(var1 = this.loadCRLEntries()).isEmpty() ? Collections.unmodifiableSet(var1) : null;
   }

   public byte[] getTBSCertList() {
      try {
         return this.c.getTBSCertList().getEncoded("DER");
      } catch (IOException var1) {
         throw new CRLException(var1.toString());
      }
   }

   public byte[] getSignature() {
      return this.c.getSignature().getBytes();
   }

   public String getSigAlgName() {
      return this.sigAlgName;
   }

   public String getSigAlgOID() {
      return this.c.getSignatureAlgorithm().getObjectId().getId();
   }

   public byte[] getSigAlgParams() {
      byte[] var3;
      if ((var3 = this.sigAlgParams) != null) {
         int var1;
         byte[] var2;
         byte[] var10000 = var2 = new byte[var1 = var3.length];
         System.arraycopy(var3, 0, var2, 0, var1);
         return var10000;
      } else {
         return null;
      }
   }

   public String toString() {
      StringBuffer var1;
      StringBuffer var10000 = var1 = new StringBuffer;
      var1.<init>();
      String var2 = System.getProperty("line.separator");
      var1.append("              Version: ").append(this.getVersion()).append(var2);
      var1.append("             IssuerDN: ").append(this.getIssuerDN()).append(var2);
      var1.append("          This update: ").append(this.getThisUpdate()).append(var2);
      var1.append("          Next update: ").append(this.getNextUpdate()).append(var2);
      var1.append("  Signature Algorithm: ").append(this.getSigAlgName()).append(var2);
      byte[] var3 = this.getSignature();
      var10000.append("            Signature: ").append(new String(Hex.encode(var3, 0, 20))).append(var2);

      for(int var4 = 20; var4 < var3.length; var4 += 20) {
         if (var4 < var3.length - 20) {
            var1.append("                       ").append(new String(Hex.encode(var3, var4, 20))).append(var2);
         } else {
            var1.append("                       ").append(new String(Hex.encode(var3, var4, var3.length - var4))).append(var2);
         }
      }

      X509Extensions var23;
      if ((var23 = this.c.getTBSCertList().getExtensions()) != null) {
         Enumeration var24;
         if ((var24 = var23.oids()).hasMoreElements()) {
            var1.append("           Extensions: ").append(var2);
         }

         label125:
         while(true) {
            while(true) {
               while(true) {
                  if (!var24.hasMoreElements()) {
                     break label125;
                  }

                  DERObjectIdentifier var5;
                  X509Extension var6;
                  if ((var6 = var23.getExtension(var5 = (DERObjectIdentifier)var24.nextElement())).getValue() != null) {
                     DERObjectIdentifier var25 = var5;
                     byte[] var7 = var6.getValue().getOctets();
                     ASN1InputStream var8;
                     var8 = new ASN1InputStream.<init>(var7);
                     var1.append("                       critical(").append(var6.isCritical()).append(") ");

                     label120: {
                        boolean var10001;
                        boolean var26;
                        try {
                           var26 = var25.equals(X509Extensions.CRLNumber);
                        } catch (Exception var20) {
                           var10001 = false;
                           break label120;
                        }

                        if (var26) {
                           try {
                              var1.append(new CRLNumber(DERInteger.getInstance(var8.readObject()).getPositiveValue())).append(var2);
                              continue;
                           } catch (Exception var9) {
                              var10001 = false;
                           }
                        } else {
                           label116: {
                              try {
                                 var26 = var5.equals(X509Extensions.DeltaCRLIndicator);
                              } catch (Exception var19) {
                                 var10001 = false;
                                 break label116;
                              }

                              if (var26) {
                                 try {
                                    var1.append("Base CRL: " + new CRLNumber(DERInteger.getInstance(var8.readObject()).getPositiveValue())).append(var2);
                                    continue;
                                 } catch (Exception var10) {
                                    var10001 = false;
                                 }
                              } else {
                                 label112: {
                                    try {
                                       var26 = var5.equals(X509Extensions.IssuingDistributionPoint);
                                    } catch (Exception var18) {
                                       var10001 = false;
                                       break label112;
                                    }

                                    if (var26) {
                                       try {
                                          var1.append(new IssuingDistributionPoint((ASN1Sequence)var8.readObject())).append(var2);
                                          continue;
                                       } catch (Exception var11) {
                                          var10001 = false;
                                       }
                                    } else {
                                       label108: {
                                          try {
                                             var26 = var5.equals(X509Extensions.CRLDistributionPoints);
                                          } catch (Exception var17) {
                                             var10001 = false;
                                             break label108;
                                          }

                                          if (var26) {
                                             try {
                                                var1.append(new CRLDistPoint((ASN1Sequence)var8.readObject())).append(var2);
                                                continue;
                                             } catch (Exception var12) {
                                                var10001 = false;
                                             }
                                          } else {
                                             label104: {
                                                try {
                                                   var26 = var5.equals(X509Extensions.FreshestCRL);
                                                } catch (Exception var16) {
                                                   var10001 = false;
                                                   break label104;
                                                }

                                                if (var26) {
                                                   try {
                                                      var1.append(new CRLDistPoint((ASN1Sequence)var8.readObject())).append(var2);
                                                      continue;
                                                   } catch (Exception var13) {
                                                      var10001 = false;
                                                   }
                                                } else {
                                                   label139: {
                                                      try {
                                                         var10000 = var1;
                                                         var1.append(var5.getId());
                                                      } catch (Exception var15) {
                                                         var10001 = false;
                                                         break label139;
                                                      }

                                                      try {
                                                         var10000.append(" value = ").append(ASN1Dump.dumpAsString(var8.readObject())).append(var2);
                                                         continue;
                                                      } catch (Exception var14) {
                                                         var10001 = false;
                                                      }
                                                   }
                                                }
                                             }
                                          }
                                       }
                                    }
                                 }
                              }
                           }
                        }
                     }

                     var1.append(var5.getId());
                     var1.append(" value = ").append("*****").append(var2);
                  } else {
                     var1.append(var2);
                  }
               }
            }
         }
      }

      Set var21;
      if ((var21 = this.getRevokedCertificates()) != null) {
         Iterator var22 = var21.iterator();

         while(var22.hasNext()) {
            var1.append(var22.next());
            var1.append(var2);
         }
      }

      return var1.toString();
   }

   public boolean isRevoked(Certificate cert) {
      if (!cert.getType().equals("X.509")) {
         throw new RuntimeException("X.509 CRL used with non X.509 Cert");
      } else {
         TBSCertList.CRLEntry[] var3;
         if ((var3 = this.c.getRevokedCertificates()) != null) {
            BigInteger cert1 = ((X509Certificate)cert).getSerialNumber();

            for(int var2 = 0; var2 < var3.length; ++var2) {
               if (var3[var2].getUserCertificate().getValue().equals(cert1)) {
                  return true;
               }
            }
         }

         return false;
      }
   }
}
